//
// Created by dell on 24-9-23.
//

#include "ultrasonic_driver.h"
#include "gd32e23x.h"
#include "systick.h"

uint8_t speed_pwm = 30;              //bldc default pwm is 30

void led_config(void)
{
    rcu_periph_clock_enable(LED_RCU);

    gpio_mode_set(LED_PORT, GPIO_MODE_OUTPUT, GPIO_PUPD_NONE, LED_PIN);
    gpio_output_options_set(LED_PORT, GPIO_OTYPE_PP, GPIO_OSPEED_50MHZ, LED_PIN);
    gpio_bit_write(LED_PORT, LED_PIN, SET);

    rcu_periph_clock_enable(RCU_TIMER13);
    timer_deinit(RCU_TIMER13);

    timer_parameter_struct timer_initpara;
    timer_struct_para_init(&timer_initpara);
    timer_initpara.prescaler        =7199;
    timer_initpara.alignedmode      =TIMER_COUNTER_EDGE;
    timer_initpara.counterdirection =TIMER_COUNTER_UP;
    timer_initpara.period           =999;
    timer_initpara.clockdivision    =TIMER_CKDIV_DIV1;
    timer_init(LED_TIMER, &timer_initpara);

    timer_auto_reload_shadow_enable(LED_TIMER);
    timer_interrupt_enable(LED_TIMER, TIMER_INT_UP);
    nvic_irq_enable(LED_IRQ, 0);
    timer_enable(LED_TIMER);
}

void usart_config(void)
{
    rcu_periph_clock_enable(USART_GPIO_RCU);
    rcu_periph_clock_enable(USART_RCU);

    gpio_af_set(USARET_GPIO_PORT, GPIO_AF_1, GPIO_PIN_2 | GPIO_PIN_3);

    /* configure USART Tx as alternate function push-pull */
    gpio_mode_set(USARET_GPIO_PORT, GPIO_MODE_AF, GPIO_PUPD_PULLUP, USART_TX_PIN | USART_RX_PIN);
    gpio_output_options_set(USARET_GPIO_PORT, GPIO_OTYPE_PP, GPIO_OSPEED_10MHZ, USART_TX_PIN | USART_RX_PIN);

    /* USART configure */
    usart_deinit(USART0_PHY);
    usart_baudrate_set(USART0_PHY, USART_BAUDRATE);
    usart_receive_config(USART0_PHY, USART_RECEIVE_ENABLE);
    usart_transmit_config(USART0_PHY, USART_TRANSMIT_ENABLE);

    usart_enable(USART0_PHY);
}

void ultrasonic_config(void)
{
    rcu_periph_clock_enable(US_TRAN_GPIO_RCU);

    gpio_mode_set(US_TRAN_GPIO_PORT, GPIO_MODE_AF, GPIO_PUPD_NONE, US_TRAN_PIN);
    gpio_output_options_set(US_TRAN_GPIO_PORT, GPIO_OTYPE_PP, GPIO_OSPEED_50MHZ, US_TRAN_PIN);
    gpio_af_set(US_TRAN_GPIO_PORT, GPIO_AF_1, US_TRAN_PIN);

    timer_oc_parameter_struct timer_ocinitpara;
    timer_parameter_struct    timer_initpara;

    rcu_periph_clock_enable(US_TRAN_RCU);
    timer_deinit(US_TRAN_TIMER);

    timer_struct_para_init(&timer_initpara);
    timer_initpara.prescaler          = 0;
    timer_initpara.alignedmode        = TIMER_COUNTER_EDGE;
    timer_initpara.counterdirection   = TIMER_COUNTER_UP;
    timer_initpara.period             = 239;
    timer_initpara.clockdivision      = TIMER_CKDIV_DIV1;
    timer_init(US_TRAN_TIMER, &timer_initpara);

    timer_channel_output_struct_para_init(&timer_ocinitpara);
    timer_ocinitpara.outputstate  =TIMER_CCX_ENABLE;
    timer_ocinitpara.outputnstate =TIMER_CCXN_DISABLE;
    timer_ocinitpara.ocpolarity   =TIMER_OC_POLARITY_HIGH;
    timer_ocinitpara.ocnpolarity  =TIMER_OCN_POLARITY_HIGH;
    timer_ocinitpara.ocidlestate  =TIMER_OC_IDLE_STATE_LOW;
    timer_ocinitpara.ocnidlestate =TIMER_OCN_IDLE_STATE_LOW;
    timer_channel_output_config(US_TRAN_TIMER, US_TRAN_CH, &timer_ocinitpara);

    timer_channel_output_pulse_value_config(US_TRAN_TIMER, US_TRAN_CH, 120);
    timer_channel_output_mode_config(US_TRAN_TIMER, US_TRAN_CH, TIMER_OC_MODE_PWM0);
    timer_auto_reload_shadow_enable(US_TRAN_TIMER);

    timer_interrupt_enable(US_TRAN_TIMER, TIMER_INT_UP);
    // nvic_irq_enable(TIMER2_IRQn, 1);
    // timer_enable(TIMER2);
}

void ultrasonic_peripheral_config(void) {
    led_config();
    usart_config();
    ultrasonic_config();
}

void ultrasonic_pwm_out_cycles(uint8_t cycles) {
    uint8_t current_cycle = 0;

    timer_channel_output_pulse_value_config(US_TRAN_TIMER, US_TRAN_CH, 120);
    timer_channel_output_mode_config(US_TRAN_TIMER, US_TRAN_CH, TIMER_OC_MODE_PWM1);
    timer_enable(US_TRAN_TIMER);

    while (current_cycle < cycles)
    {
        while (!timer_interrupt_flag_get(US_TRAN_TIMER, TIMER_INT_FLAG_UP));
        timer_interrupt_flag_clear(US_TRAN_TIMER, TIMER_INT_FLAG_UP);
        current_cycle ++;
    }
    // delay_nop();
    timer_disable(US_TRAN_TIMER);
    // if(gpio_output_bit_get(GPIOB, GPIO_PIN_1) == SET)
    // {
    //     gpio_bit_reset(GPIOB, GPIO_PIN_1);
    // }
}
